require "Window"
 
local KiruTweaks = {} 

local versionNumber = "v0.981"

local optionWindowSize = 495

--some keys don't match their control character and the special keys
--don't have a printable description so.. table
local tblSpecialKeys = {
	[8] = "BK",
	[9] = "TAB",
	[13] = "ENT",
	[16] = "SHF",
	[17] = "CTR",
	[27] = "ESC",
	[32] = "SPA",
	[33] = "PUP",
	[34] = "PDN",
	[35] = "END",
	[36] = "HOM",
	[37] = "LFT",
	[38] = "UP",
	[39] = "RT",
	[40] = "DWN",
	[45] = "INS",
	[46] = "DEL",
	[106] = "*",
	[107] = "+",
	[109] = "-",
	[110] = ".",
	[111] = "/",
	[112] = "F1",
	[113] = "F2",
	[114] = "F3",
	[115] = "F4",
	[116] = "F5",
	[117] = "F6",
	[118] = "F7",
	[119] = "F8",
	[120] = "F9",
	[121] = "F10",
	[122] = "F11",
	[123] = "F12",
	[144] = "NUM",
	[145] = "SCR",
	[185] = ":",
	[187] = "=",
	[188] = ",",
	[189] = "-",
	[190] = ".",
	[191] = "/",
	[192] = "`",
	[219] = "[",
	[220] = "|",
	[221] = "]"
}
 
function KiruTweaks:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.Preferences = {}
	self.Option = {}

    return o
end

function KiruTweaks:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "KiruTweaks"
	local tDependencies = {
			--Pie, cookies
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
function KiruTweaks:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("KiruTweaks.xml")
--	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	Apollo.RegisterSlashCommand("kt", "OnSlash", self)
	Apollo.RegisterSlashCommand("ktqt", "OnQTSlash", self)
	Apollo.RegisterSlashCommand("kib", "OnBusySlash", self)
	
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("ShowKiruTweaks", "OnSlash", self)
	
	Apollo.RegisterEventHandler("GenericEvent_InitializeSchematicsTree", "TradeOpened", self)
	
	--to hide the housing thing
	Apollo.RegisterEventHandler("ChangeWorld", "OnChangeWorld", self)

	-- a hack to run after everything is loaded.
	self.HackInit = 1
	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnNextFrame", self)

end

-----------------------------------------------------------------------------------------------
-- a hack to run after everything is loaded.
function KiruTweaks:OnNextFrame()
	if Apollo == nil then return end
	if GameLib.GetPlayerUnit() == nil then return end
	if HousingLib == nil then return end
	
	if self.Preferences.HideHouse == true then
		local isHouse = HousingLib.IsHousingWorld()
		local remodel = Apollo.GetAddon("HousingRemodel")
		if isHouse == true and remodel ~= nil and remodel.wndConfigure ~= nil then
			self:AddHookHouseRemodel()
		end
	end
	
	if self.Change == true then self.Change = nil end
	if self.HackInit == 1 then
		self.HackInit = nil
		self.Attempts = 0
		self.MaxAttempts = 20 --check for addons for 10 seconds
		self.Timer = ApolloTimer.Create(0.5,true,"InitTimer",self)
	end
	
	--turn off the frame spam
	Apollo.RemoveEventHandler("VarChange_FrameCount",self)
end

-----------------------------------------------------------------------------------------------
-- post init get addons and whatnot
function KiruTweaks:InitTimer()
	self.Attempts = self.Attempts + 1
	
	if self.QT == nil then
		local QT = Apollo.GetAddon("ObjectiveTracker")
		if QT ~= nil then self.QT = QT.wndMain end
	end
	if self.CSI == nil then
		self.CSI = Apollo.GetAddon("CSI")
	end
	if self.PlayerName == nil then
		local player = GameLib.GetPlayerUnit()
		if player ~= nil then self.PlayerName = player:GetName() end
	end
	if self.Commodity == nil then
		self.Commodity = Apollo.GetAddon("MarketplaceCommodity")
	end
	if self.Auction == nil then
		self.Auction = Apollo.GetAddon("MarketplaceAuction")
	end
		
	if (self.Attempts == self.MaxAttempts) or (self.QT ~= nil 
		and self.CSI ~= nil 
		and self.PlayerName ~= nil 
		and self.Commodity ~= nil
		and self.Auction ~= nil) then
		self.Timer:Stop()
		self.Timer = nil
		self:PostInit()
	end 
end

function KiruTweaks:PostInit()
	--read in from the config data
	local FR = false
	if self.Preferences.FromRestore ~= nil then FR = self.Preferences.FromRestore end
	if FR then FR = FR.Preferences else FR = {} end
	if FR == nil then FR = {} end
	
	local Prefs = self.Preferences
	
	Prefs.YesNo = 				FR.YesNo
	Prefs.ItemPreview = 		FR.ItemPreview
	Prefs.QuestTracker = 		FR.QuestTracker
	Prefs.QTAlways = 			FR.QTAlways
	Prefs.QTCombat = 			FR.QTCombat
	Prefs.AutoReplyBusy = 		FR.AutoReplyBusy
	Prefs.AutoReplyCombat = 	FR.AutoReplyCombat
	Prefs.AutoReplyCombatTarget = FR.AutoReplyCombatTarget
	Prefs.KeybindNo = 			FR.KeybindNo
	Prefs.KeybindYes = 			FR.KeybindYes
	Prefs.ShowStartup = 		FR.ShowStartup
	Prefs.HideHouse = 			FR.HideHouse
	Prefs.ItemLinkSchem = 		FR.ItemLinkSchem
	Prefs.AddWhisper = 			FR.AddWhisper
		
	if Prefs.YesNo == nil then Prefs.YesNo = false end
	if Prefs.ItemPreview == nil then Prefs.ItemPreview = false end
	if Prefs.QuestTracker == nil then Prefs.QuestTracker = false end
	if Prefs.QTAlways == nil and Prefs.QTCombat == nil then Prefs.QTAlways = true end
	if Prefs.QTAlways == nil then Prefs.QTAlways = false end
	if Prefs.QTCombat == nil then Prefs.QTCombat = false end	
	if Prefs.AutoReplyBusy == nil then Prefs.AutoReplyBusy = false end
	if Prefs.AutoReplyCombat == nil then Prefs.AutoReplyCombat = false end
	if Prefs.AutoReplyCombatTarget == nil then Prefs.AutoReplyCombatTarget = false end
	if Prefs.ShowStartup == nil then Prefs.ShowStartup = true end
	if Prefs.HideHouse == nil then Prefs.HideHouse = false end
	if Prefs.ItemLinkSchem == nil then Prefs.ItemLinkSchem = false end
	if Prefs.AddWhisper == nil then Prefs.AddWhisper = false end
	
	Prefs.BusyMessage = 	FR.BusyMessage or "I am currently busy and will reply as soon as I can!"
	Prefs.CombatMessage = 	FR.CombatMessage or "I am currently in Combat and will reply as soon as I can."			
	
	if Prefs.QuestTracker == true and (Prefs.QTAlways == false and Prefs.QTAlways == false) then Prefs.QTCombat = true end 
	
	local tweakStartMsg = ""
	if Prefs.QuestTracker == true then
		if self.QT == nil then
			tweakStartMsg = tweakStartMsg .. " T1: Cannot find the Objective Tracker to hide it."		
		else
			if Prefs.QTAlways == true then
				self.QT:Show(false)
				tweakStartMsg = tweakStartMsg .. " T1: Objective Tracker should be hidden."
			else
				tweakStartMsg = tweakStartMsg .. " T1: Objective Tracker should hide during Combat."					
			end		
		end
	end
	
	if Prefs.AutoReplyCombat == true then
		if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
		tweakStartMsg = tweakStartMsg .. " T2: Should auto-reply when you are in Combat."
		if Prefs.AutoReplyCombatTarget == true then tweakStartMsg = tweakStartMsg .. " (With current target)" end	
	end	
		
	if Prefs.AutoReplyBusy == true then
		if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
		tweakStartMsg = tweakStartMsg .. " T3: Should auto-reply when you are in Busy Mode (/kib)."	
	end
	
	if Prefs.YesNo == true then
		if self.CSI == nil then
			if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
			tweakStartMsg = tweakStartMsg .. " T4: Cannot find CSI to add keybinds."		
		else
			if self.Preferences.KeybindYes == nil and self.Preferences.KeybindNo == nil then
				if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
				tweakStartMsg = tweakStartMsg .. " T4: No Keybinds are defined for the Yes/No prompt."
			else
				local hook = self:AddHookYesNo()
				if hook == true then
					if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
					tweakStartMsg = tweakStartMsg .. " T4: Keybinds should be added to Yes/No prompt."
				else
					if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
					tweakStartMsg = tweakStartMsg .. " T4: There was a problem connecting to CSI for the Yes/No prompt."
				end			
			end
		end	
	end
		
	if Prefs.ItemPreview == true then
		local hook = self:AddHookPopup()
		if hook == true then
			if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
			tweakStartMsg = tweakStartMsg .. " T5: Item Preview window should be closeable with ESC."
		else
			if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
			tweakStartMsg = tweakStartMsg .. " T5: There was a problem connecting to the item preview window."
		end			
	end	
	
	if Prefs.HideHouse == true then
		if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
		tweakStartMsg = tweakStartMsg .. " T6: Will attempt to hide the House UI Header Piece."
		local isHouse = HousingLib.IsHousingWorld()
		local remodel = Apollo.GetAddon("HousingRemodel")
		if isHouse == true and remodel ~= nil and remodel.wndConfigure ~= nil then
			self:AddHookHouseRemodel()
		end
	end
	
	if Prefs.ItemLinkSchem == true then
		if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
		tweakStartMsg = tweakStartMsg .. " T7: Will attempt to make Tradeskill Schematic Sub-Recipes linkable."
	end

	if Prefs.AddWhisper == true then
		local hook = self:AddHookContextMenu()
		if hook == true then
			if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
			tweakStartMsg = tweakStartMsg .. " T8: Should be able to Whisper your Account Friends."
		else
			if tweakStartMsg ~= "" then tweakStartMsg = tweakStartMsg .. "\n" end
			tweakStartMsg = tweakStartMsg .. " T8: There was a problem connecting to the context menu."
		end			
	end
	
	------------------------------------------------------
				
	if tweakStartMsg == "" then tweakStartMsg = "No special actions to perform." end

	if Prefs.ShowStartup == true then
		Print("KiruTweaks Startup Report:\n" .. tweakStartMsg .. "\nType /kt or open KiruTweaks from the menu.")
	end
	
	Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)

	self.InCombat = false
	self.IsBusy = false
end

-----------------------------------------------------------------------------------------------
-- default system events

function KiruTweaks:OnSave(eType)
	--account-level saving
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then return end
	
	local saveData = {}

	local preferences = self.Preferences
	
	--if the option window is open at time of saving, save its current position
	if self.Option ~= nil and self.Option.Window ~= nil then
		l,t,r,b = self.Option.Window:GetAnchorOffsets()
		saveData.OptionWindowPos = { l = l, t = t, r = r, b = b}
	elseif preferences.FromRestore ~= nil then
		--otherwise persist its last known location
		saveData.OptionWindowPos = preferences.FromRestore.OptionWindowPos			
	end

	--save everything else		
	saveData.Preferences = self.Preferences
	--dont need to cause a recursive loop
	saveData.Preferences.FromRestore = nil
	return saveData
end

function KiruTweaks:OnRestore(eType, tSavedData)
	--account-level saving
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then return end
	self.Preferences.FromRestore = tSavedData	
end

function KiruTweaks:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "KiruTweaks", {"ShowKiruTweaks", "", ""})
end

function KiruTweaks:OnConfigure()
	self:OnSlash()
end

-----------------------------------------------------------------------------------------------
-- slashes

function KiruTweaks:OnSlash(command, args)
	if self.Option == nil or self.Option.Window == nil then
		self:CreateOptionWindow()
	else
		self:Button_HideOption()
	end	
end

function KiruTweaks:OnQTSlash(command, args)
	if self.QT == nil then Print("KiruTweaks: Not connected to the Objective Tracker.") return end

	local visible = true
	if args == "" then visible = not self.QT:IsVisible() end  --toggle
	if args == "off" then visible = false end
	
	self.QT:Show(visible,true)
	self.Preferences.QuestTracker = not visible

	if visible == false then
		self.Preferences.QTCombat = false
		self.Preferences.QTAlways = true
		Print("KiruTweaks: Attempting to Hide the Objective Tracker.")
	else
		Print("KiruTweaks: Attempting to Show the Objective Tracker.")	
	end
	if self.Option ~= nil and self.Option.Window ~= nil then	
		self.Option.Window:FindChild("Button_QuestTracker"):SetCheck(self.Preferences.QuestTracker)
		self.Option.Window:FindChild("Radio_QTAlways"):Enable(self.Preferences.QuestTracker)
		self.Option.Window:FindChild("Radio_QTAlways"):SetCheck(self.Preferences.QTAlways)
		self.Option.Window:FindChild("Radio_QTCombat"):SetCheck(self.Preferences.QTCombat)
		self.Option.Window:FindChild("Radio_QTCombat"):Enable(self.Preferences.QuestTracker)
	end
end

function KiruTweaks:OnBusySlash(command, args)
	if args == "" then 
		self.IsBusy = not self.IsBusy
		if self.IsBusy then Print("KiruTweaks: Busy Mode ON.  Type /kib by itself to turn it off.") else Print("KiruTweaks: Busy Mode OFF.") end
		return
	end
	
	self.Preferences.BusyMessage = args
	Print("KiruTweaks: Your 'Busy Mode' Message Will Read: (Auto Reply) " .. self.IsBusyMessage)
end

-----------------------------------------------------------------------------------------------
-- events

function KiruTweaks:OnChangeWorld()
	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnNextFrame", self)
end

function KiruTweaks:OnSystemKeyDown(iKey)
	if self.Preferences.YesNo == true and self.YesNoWindow then
		if self.Preferences.KeybindYes == iKey then
			self.YesNoWindow = nil
			self:EnableKeydown(false)
			self.Hooks["YesNoYes"](self.CSI, self.YesNoYesBtn, self.YesNoYesBtn)
		end
		if self.Preferences.KeybindNo == iKey then
			self.YesNoWindow = nil
			self:EnableKeydown(false)
			self.Hooks["YesNoNo"](self.CSI, self.YesNoNoBtn, self.YesNoNoBtn)	
		end	
	end
	if self.CaptureKeybindYes then
		if self.Option == nil or self.Option.Window == nil then
			self.CaptureKeybindYes = nil
			self:EnableKeydown(false)
			return
		end
		if self.Preferences.KeybindNo and iKey == self.Preferences.KeybindNo then return end
		self.Preferences.KeybindYes = iKey
		local button = self.Option.Window:FindChild("Button_CaptureYes")
		local textbox = self.Option.Window:FindChild("KeybindYesEditbox")
		button:Enable(true)
		button:SetText("Capture Key")
		local keytext = tblSpecialKeys[iKey] or string.char(iKey)	
		textbox:SetText(keytext)
		self.CaptureKeybindYes = nil
		if self.Preferences.YesNo == true then 
			local hooked = self:AddHookYesNo()
			if hooked == true then
				Print("KiruTweaks: There should be a keybind on the yes prompt now.")			
			else
				Print("KiruTweaks: There was a problem connecting to CSI for the yes/no prompt.")
			end		
		end
		self:EnableKeydown(false)
		return
	end
	if self.CaptureKeybindNo then
		if self.Option == nil or self.Option.Window == nil then
			self.CaptureKeybindNo = nil
			self:EnableKeydown(false)
			return
		end
		if self.Preferences.KeybindYes and iKey == self.Preferences.KeybindYes then return end
		self.Preferences.KeybindNo = iKey
		local button = self.Option.Window:FindChild("Button_CaptureNo")
		local textbox = self.Option.Window:FindChild("KeybindNoEditbox")
		button:Enable(true)
		button:SetText("Capture Key")
		local keytext = tblSpecialKeys[iKey] or string.char(iKey)	
		textbox:SetText(keytext)
		if self.Preferences.YesNo == true then
			local hooked = self:AddHookYesNo()
			if hooked == true then
				Print("KiruTweaks: There should be a keybind on the no prompt now.")			
			else
				Print("KiruTweaks: There was a problem connecting to CSI for the yes/no prompt.")
			end
		end
		self.CaptureKeybindNo = nil
		self:EnableKeydown(false)
		return	
	end	
end

function KiruTweaks:OnEnteredCombat(unit, inCombat)
	if unit == nil then return end
	if self.PlayerName == nil then return end
	if unit:GetName() ~= self.PlayerName then return end

	self.InCombat = inCombat
	
	if self.Preferences.QuestTracker == true then
		if self.Preferences.QTCombat == true and self.QT ~= nil then
			self.QT:Show(not self.InCombat)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- UI Stuff

function KiruTweaks:CreateOptionWindow()
	self.Option = {}
    self.Option.Window = Apollo.LoadForm(self.xmlDoc, "Option", nil, self)
	if self.Option.Window == nil then
		Apollo.AddAddonErrorText(self, "Could not load the option window for some reason.")
		return
	end 
	
	--set the version number in title
	self.Option.Window:FindChild("Title"):SetText("KiruTweaks " .. versionNumber)

	--read in from the config data
	if self.Preferences.FromRestore ~= nil then
		local FromRestore = self.Preferences.FromRestore
		local anchors = FromRestore.OptionWindowPos
		if anchors then
			if (math.abs(anchors.b - anchors.t) ~= optionWindowSize) then anchors.b = anchors.t + optionWindowSize end
			self.Option.Window:SetAnchorOffsets(anchors.l, anchors.t, anchors.r, anchors.b)
		end	
		if self.Preferences.KeybindYes ~= nil then 
			local textbox = self.Option.Window:FindChild("KeybindYesEditbox")
			local keytext = tblSpecialKeys[self.Preferences.KeybindYes]
			if keytext == nil then keytext = string.char(self.Preferences.KeybindYes) end
			textbox:SetText(keytext)
		end
		if self.Preferences.KeybindNo ~= nil then 
			local textbox = self.Option.Window:FindChild("KeybindNoEditbox")
			local keytext = tblSpecialKeys[self.Preferences.KeybindNo]
			if keytext == nil then keytext = string.char(self.Preferences.KeybindNo) end
			textbox:SetText(keytext)
		end
	end	
	--checkboxes
	self:UISetCheckbox(self.Option.Window, "ItemPreview", self.Preferences.ItemPreview)
	self:UISetCheckbox(self.Option.Window, "QuestTracker", self.Preferences.QuestTracker)
	self:UISetCheckbox(self.Option.Window, "YesNo", self.Preferences.YesNo)
	self:UISetCheckbox(self.Option.Window, "AutoReply", self.Preferences.AutoReplyBusy)
	self:UISetCheckbox(self.Option.Window, "AutoReplyCombat", self.Preferences.AutoReplyCombat)
	self:UISetCheckbox(self.Option.Window, "AutoReplyCombatTarget", self.Preferences.AutoReplyCombatTarget)
	self:UISetCheckbox(self.Option.Window, "ShowStartup", self.Preferences.ShowStartup)
	self:UISetCheckbox(self.Option.Window, "HideHouse", self.Preferences.HideHouse)
	self:UISetCheckbox(self.Option.Window, "ItemLinkSchem", self.Preferences.ItemLinkSchem)
	self:UISetCheckbox(self.Option.Window, "AddWhisper", self.Preferences.AddWhisper)
		
	self.Option.Window:FindChild("Button_AutoReplyCombatTarget"):Enable(self.Preferences.AutoReplyCombat == true)
	
	--enable/disable stuff
	self.Option.Window:FindChild("Radio_QTAlways"):SetCheck(self.Preferences.QTAlways == true)
	self.Option.Window:FindChild("Radio_QTCombat"):SetCheck(self.Preferences.QTCombat == true)
	if self.Preferences.QuestTracker ~= true then 
		self.Option.Window:FindChild("Radio_QTAlways"):Enable(false)
		self.Option.Window:FindChild("Radio_QTCombat"):Enable(false)
	end
	
	self.Option.Window:FindChild("EditBox_Busy"):SetText(self.Preferences.BusyMessage)
	self.Option.Window:FindChild("EditBox_Combat"):SetText(self.Preferences.CombatMessage)
					
	self.Option.Window:Show(true, true)	
end

function KiruTweaks:UISetCheckbox(parentWindow, buttonName, check)
	parentWindow:FindChild("Button_" ..buttonName):SetCheck(check)
end

function KiruTweaks:Button_HideOption()
	--overwrite the option window position then destroy
	if self.Preferences.FromRestore == nil then self.Preferences.FromRestore = {} end
	local l,t,r,b = self.Option.Window:GetAnchorOffsets()
	self.Preferences.FromRestore.OptionWindowPos = { l = l, t = t, r = r, b = b}
	self.Option.Window:Destroy()
	self.Option = {}
end

-----------------------------------------------------------------------------------------------
-- catch the tradeskill window

function KiruTweaks:TradeOpened()
	if self.TradeskillSchematics == nil then
		self.TempTimer = ApolloTimer.Create(0.2, true, "TradeTimer", self)
	end
end

function KiruTweaks:TradeTimer()
	self.TradeskillSchematics = Apollo.GetAddon("TradeskillSchematics")	
	if self.Preferences.ItemLinkSchem == true then self:AddHookSchematic() end
	self.TempTimer:Stop()
	self.TempTimer = nil
end                                                 
                                                        
-----------------------------------------------------------------------------------------------
-- hooks:
function KiruTweaks:AddHookContextMenu()
	self:RemoveHookContextMenu()
	local context = Apollo.GetAddon("ContextMenuPlayer")
	if context == nil then return false end
	if self.Hooks == nil then self.Hooks = {} end
	if self.Hooks["ContextAll"] == nil then
		self.Hooks["ContextAll"] = context.RedrawAll
		context.RedrawAll = function(contx)
			contx:KiruAddWhisper(contx)
			--let original run
			self.Hooks["ContextAll"](contx)
		end		
	end
	
	if self.Hooks["ContextFriend"] == nil then
		self.Hooks["ContextFriend"] = context.RedrawAllFriend
		context.RedrawAllFriend = function(contx)
			contx:KiruAddWhisper(contx)
			--let original run
			self.Hooks["ContextFriend"](contx)
		end
	end	
	
	context.KiruAddWhisper = function(contx)
		if not contx.wndMain or not contx.wndMain:IsValid() then return end		
		local bCrossFaction = contx.bCrossFaction
		local tFriend = contx.tFriend
		local tAccountFriend = contx.tAccountFriend
		local unitPlayer = GameLib.GetPlayerUnit()
		local bCanWhisper = tFriend ~= nil and tFriend.fLastOnline == 0 and not tFriend.bIgnore and tFriend.nFactionId == unitPlayer:GetFaction()
		local bCanAccountWisper = tAccountFriend ~= nil and tAccountFriend.arCharacters and tAccountFriend.arCharacters[1] ~= nil
		if bCanAccountWisper then
			bCanWhisper = tAccountFriend.arCharacters[1] ~= nil
				and tAccountFriend.arCharacters[1].strRealm == GameLib.GetRealmName()
				and tAccountFriend.arCharacters[1].nFactionId == unitPlayer:GetFaction()
		end
		--all that for this.
		if bCanWhisper and bCanAccountWisper then
			local wndButtonList = contx.wndMain:FindChild("ButtonList")
			contx:HelperBuildRegularButton(wndButtonList, "BtnWhisper", Apollo.GetString("ContextMenu_Whisper"))
			--contx:ResizeAndRedraw()
		end	
	end
	
	return true
end

function KiruTweaks:RemoveHookContextMenu()
	if self.Hooks["ContextAll"] ~= nil then
		local context = Apollo.GetAddon("ContextMenuPlayer")
		if context ~= nil then context.RedrawAll = self.Hooks["ContextAll"] end
		if context ~= nil then context.RedrawAllFriend = self.Hooks["ContextFriend"] end
		self.Hooks["ContextAll"] = nil
		self.Hooks["ContextFriend"] = nil
	end
end

function KiruTweaks:AddHookHouseRemodel()
	self:RemoveHookHouseRemodel()
	local remodel = Apollo.GetAddon("HousingRemodel")
	if remodel == nil then return end
	if remodel.wndConfigure == nil then return end
	remodel.wndConfigure:Show(false)
	if self.Hooks == nil then self.Hooks = {} end
	if self.Hooks["HouseRemodel"] == nil then
		self.Hooks["HouseRemodel"] = remodel.HelperShowHeader
		remodel.HelperShowHeader = function(remodel)
				if remodel.wndConfigure == nil then return end
				remodel.wndConfigure:Show(false)
			end
	end
end

function KiruTweaks:RemoveHookHouseRemodel()
	if self.Hooks["HouseRemodel"] ~= nil then
		local remodel = Apollo.GetAddon("HousingRemodel")
		if remodel ~= nil then remodel.HelperShowHeader = self.Hooks["HouseRemodel"] end
		self.Hooks["HouseRemodel"] = nil
	end
end

function KiruTweaks:AddHookSchematic()
	if self.TradeskillSchematics == nil then 
		--make one more attempt at it
		self.TradeskillSchematics = Apollo.GetAddon("TradeskillSchematics")
	end
	
	if self.TradeskillSchematics == nil then return false end
	if self.Hooks == nil then self.Hooks = {} end
	
	if self.Hooks["Schematics"] == nil then
		self.Hooks["Schematics"] = self.TradeskillSchematics.DrawSchematic
		self.TradeskillSchematics.DrawSchematic = function(schem, tSchematic)
			--let original run
			self.Hooks["Schematics"](schem, tSchematic)
			local tSchematicInfo = CraftingLib.GetSchematicInfo(tSchematic.nSchematicId)
			local wndSchem = self.TradeskillSchematics.wndMain:FindChild("RightSide")
			--tap main recipe.. no longer needed
--			local icon = wndSchem:FindChild("SchematicIcon")
--			icon:SetData(tSchematicInfo.itemOutput)			
--			icon:RemoveEventHandler("MouseButtonDown",self.TradeskillSchematics)
--			icon:AddEventHandler("MouseButtonDown","KiruMouseDown",self.TradeskillSchematics)
			
			for key, tSubrecipe in pairs(tSchematicInfo.tSubRecipes) do
				local parent = wndSchem:FindChild("SubrecipesListScroll")
				
				--loop kids, find it
				for idx, win in pairs(parent:GetChildren()) do
					local test = win:FindChild("SubrecipesLeftName"):GetText()
					if test == tSubrecipe.itemOutput:GetName() then
						local icon = win:FindChild("SubrecipesLeftIcon")
						icon:SetData(tSubrecipe.itemOutput)
						icon:RemoveEventHandler("MouseButtonDown",self.TradeskillSchematics)
						icon:AddEventHandler("MouseButtonDown","KiruMouseDown",self.TradeskillSchematics)
					end
				end
			end		
		end
		
		self.TradeskillSchematics.KiruMouseDown = function(schem, wndHandler, wndControl, eMouseButton)
			if (not eMouseButton == GameLib.CodeEnumInputMouse.Right) then return end
			--if (not Apollo.IsShiftKeyDown()) then return end
			local item = wndControl:GetData()
			Event_FireGenericEvent("ItemLink", item)
		end
	end
	return true
end

function KiruTweaks:RemoveHookSchematic()
	if self.TradeskillSchematics == nil then return end
	if self.Hooks["Schematics"] ~= nil then
		self.TradeskillSchematics.DrawSchematic = self.Hooks["Schematics"]
		self.TradeskillSchematics.KiruMouseDown = nil
		self.Hooks["Schematics"] = nil
	end
end

function KiruTweaks:AddHookPopup()
	if Tooltip == nil then return false end	

	if self.Hooks == nil then self.Hooks = {} end
	
	if self.Hooks["Tooltip"] == nil then
		self.Hooks["Tooltip"] = Tooltip.GetItemTooltipForm
		--luaCaller, wndParent, itemSource, tFlags, nCount
		Tooltip.GetItemTooltipForm = function(luaCaller, wndParent, itemSource, tFlags, nCount)
			local parent = (tFlags and tFlags.wndParent) or nil
			if parent ~= nil then
				local name = parent:GetName()
				if name == "TooltipWindow" then 
					parent:SetStyle("Escapable",true)
					local chatlog = Apollo.GetAddon("ChatLog")
					chatlog.ToolTipESCClose = function(wndHandler, wndControl) 
						local itemData = wndControl:GetData()
						for idx, wndCur in pairs(chatlog.twndItemLinkTooltips) do
							if wndCur:GetData() == itemData then
								table.remove(chatlog.twndItemLinkTooltips, idx)
							end
						end
					end
					parent:AddEventHandler("WindowClosed","ToolTipESCClose",chatlog)
				end
			end
			return self.Hooks["Tooltip"](luaCaller, wndParent, itemSource, tFlags, nCount)
		end
	end
	
	return true
end

function KiruTweaks:RemoveHookPopup()
	if self.Hooks["Tooltip"] ~= nil then
		Tooltip.GetItemTooltipForm = self.Hooks["Tooltip"]
		self.Hooks["Tooltip"] = nil
	end
end

function KiruTweaks:AddHookYesNo()
	if self.CSI == nil then return false end
	if self.Preferences.YesNo ~= true then return false end
	if self.Preferences.KeybindNo == nil and self.Preferences.KeybindYes == nil then return false end
	
	if self.Hooks == nil then self.Hooks = {} end
	if self.Hooks["YesNo"] == nil then
		self.Hooks["YesNo"] = self.CSI.BuildYesNo

		self.CSI.BuildYesNo = function(csi, tActiveCSI)
			--let it build
			self.Hooks["YesNo"](csi, tActiveCSI)
			
			local prompt = self.CSI.wndProgress
			if prompt == nil then 
				self.YesNoWindow = nil
				self:RemoveSubHookYesNo()
				return 
			end

			local yesbtn = prompt:FindChild("YesButton")
			local nobtn = prompt:FindChild("NoButton")
			if yesbtn == nil then 
				self.YesNoWindow = nil
				self:RemoveSubHookYesNo()
				return 
			end
			
			self:RemoveSubHookYesNo()
			self.YesNoWindow = 1
			
			local yesBind = ""
			local noBind = ""
			if self.Preferences.KeybindYes then yesBind = tblSpecialKeys[self.Preferences.KeybindYes] or string.char(self.Preferences.KeybindYes) end
			if self.Preferences.KeybindNo then noBind = tblSpecialKeys[self.Preferences.KeybindNo] or string.char(self.Preferences.KeybindNo) end

			if yesBind ~= "" then yesbtn:SetText("(" .. yesBind .. ") " .. yesbtn:GetText()) end
			if noBind ~= "" then nobtn:SetText("(" .. noBind .. ") " .. nobtn:GetText()) end

			self.YesNoYesBtn = yesbtn
			self.YesNoNoBtn = nobtn
			
			self:EnableKeydown(true)

			--clean it up
			self.Hooks["YesNoYes"] = self.CSI.OnYesNo_YesPicked
			self.Hooks["YesNoNo"] = self.CSI.OnYesNo_NoPicked
									
			self.CSI.OnYesNo_NoPicked = function(csi, wndHandler, wndControl)
				self:EnableKeydown(false)
				self.Hooks["YesNoNo"](self.CSI, self.YesNoNoBtn, wndControl)
			end
			self.CSI.OnYesNo_YesPicked = function(csi, wndHandler, wndControl)
				self:EnableKeydown(false)
				self.Hooks["YesNoYes"](self.CSI, self.YesNoYesBtn, wndControl)
			end			
		end
	end
	
	return true
end

function KiruTweaks:RemoveHookYesNo()
	if self.Hooks["NoYes"] ~= nil then
		if self.CSI ~= nil then self.CSI.BuildYesNo = self.Hooks["NoYes"] end
		self.Hooks["NoYes"] = nil
	end
	self:RemoveSubHookYesNo()
end

function KiruTweaks:RemoveSubHookYesNo()
	if self.Hooks["YesNoYes"] ~= nil then
		if self.CSI ~= nil then self.CSI.OnYesNo_YesPicked = self.Hooks["YesNoYes"] end
		self.Hooks["YesNoYes"] = nil
	end
	if self.Hooks["YesNoNo"] ~= nil then
		if self.CSI ~= nil then self.CSI.OnYesNo_NoPicked = self.Hooks["YesNoNo"] end
		self.Hooks["YesNoNo"] = nil
	end
end

function KiruTweaks:EnableKeydown(enable)
	if enable then
		Apollo.RegisterEventHandler("SystemKeyDown","OnSystemKeyDown",self)	
	else
		Apollo.RemoveEventHandler("SystemKeyDown",self)
	end
end

-----------------------------------------------------------------------------------------------
-- autoreply stuff

function KiruTweaks:OnChatMessage(channel, message)
	--ignore ourselves
	if message.bSelf == true then return end
	
	if self.Preferences.AutoReplyBusy ~= true and self.Preferences.AutoReplyCombat ~= true then return end
	
    local eChannelType = channel:GetType()

   	if eChannelType == ChatSystemLib.ChatChannel_Whisper or eChannelType == ChatSystemLib.ChatChannel_AccountWhisper then
		
		local tempMessage = ""
		for idx, segment in pairs(message.arMessageSegments) do
			tempMessage = tempMessage .. segment.strText
		end	
		
		--don't reply to autoreplies...
		if self:StringStarts(tempMessage, "(Auto Reply)") == true then return end
		--don't reply to the AFK message...
		if tempMessage == "Currently away from keyboard." then return end

		local sender = message.strSender
		local realm = message.strRealmName
				
		local msgTo = sender
		if realm ~= nil and realm ~= "" then msgTo = msgTo .. "@" .. realm end
		if self.IsBusy == true and self.Preferences.AutoReplyBusy == true then
			ChatSystemLib.Command("/w " .. msgTo .. " (Auto Reply) " .. self.Preferences.BusyMessage)
			return
		end
		if self.InCombat == true and self.Preferences.AutoReplyCombat == true then
			local addTarget = ""
			if self.Preferences.AutoReplyCombatTarget == true then
				local target = GameLib.GetTargetUnit()
				if target ~= nil then 
					local health = target:GetHealth()
					local maxhealth = target:GetMaxHealth()
					addTarget = "  Current Target: " .. target:GetName()
					if health ~= nil and maxhealth ~= nil then
						addTarget = addTarget .. " {" .. tostring(self:Round((health * 100) / maxhealth, 0)) .. "%)"
					end
				end
			end
			ChatSystemLib.Command("/w " .. msgTo .. " (Auto Reply) " .. self.Preferences.CombatMessage .. addTarget)
			return	
		end		
	end
end

-----------------------------------------------------------------------------------------------
-- helper junk

function KiruTweaks:StringStarts(input, check)
   return string.sub(input,1,string.len(check)) == check
end

---------------------------------------------------------------------------------------------------
-- Option Functions
---------------------------------------------------------------------------------------------------

function KiruTweaks:Button_ItemPreview( wndHandler, wndControl, eMouseButton )
	self.Preferences.ItemPreview = wndControl:IsChecked()
	if self.Preferences.ItemPreview then
		local hooked = self:AddHookPopup()
		if hooked == true then
			Print("KiruTweaks: Any new item detail windows should be escapable now.")			
		else
			Print("KiruTweaks: There was a problem connecting with the item detail windows.")
		end
	else
		self:RemoveHookPopup()
		Print("KiruTweaks: No longer attempting to mess with new item detail windows.")			
	end
end

function KiruTweaks:Button_QuestTracker( wndHandler, wndControl, eMouseButton )
	self.Preferences.QuestTracker = wndControl:IsChecked()
	self.Option.Window:FindChild("Radio_QTAlways"):Enable(self.Preferences.QuestTracker)
	self.Option.Window:FindChild("Radio_QTCombat"):Enable(self.Preferences.QuestTracker)

	if self.QT ~= nil then
		if self.Preferences.QuestTracker == true then
			if self.Preferences.QTAlways == true then self.QT:Show(false, true) end
		else
			self.QT:Show(true, true)
		end		
	end
end

function KiruTweaks:Radio_QTAlways( wndHandler, wndControl, eMouseButton )
	self.Preferences.QTAlways = true
	self.Preferences.QTCombat = false
	if self.Preferences.QuestTracker == true and self.QT ~= nil then self.QT:Show(false,true) end
end

function KiruTweaks:Radio_QTCombat( wndHandler, wndControl, eMouseButton )
	self.Preferences.QTCombat = true
	self.Preferences.QTAlways = false
	if self.Preferences.QuestTracker == true and self.InCombat == true and self.QT ~= nil then self.QT:Show(false,true) return end
	if self.QT ~= nil then self.QT:Show(true,true) end
end

function KiruTweaks:Button_YesNo( wndHandler, wndControl, eMouseButton )
	self.Preferences.YesNo = wndControl:IsChecked()
	if self.Preferences.YesNo then
		if self.Preferences.KeybindNo == nil and self.Preferences.KeybindYes == nil then
			Print("Choose at least one keybind.")
		else
			local hooked = self:AddHookYesNo()
			if hooked == true then
				Print("KiruTweaks: There should be keybind(s) on the yes/no prompt now.")			
			else
				Print("KiruTweaks: There was a problem connecting to CSI for the yes/no prompt.")
			end
		end
	else
		self:RemoveHookYesNo()
		Print("KiruTweaks: No longer attempting to add keybinds to yes/no prompt.")			
	end	
end

function KiruTweaks:Button_KeybindCaptureYes( wndHandler, wndControl, eMouseButton )
	self.CaptureKeybindYes = true
	self:EnableKeydown(true)
	wndControl:Enable(false)
	wndControl:SetText("Press A Key")
end

function KiruTweaks:Button_KeybindCaptureNo( wndHandler, wndControl, eMouseButton )
	self.CaptureKeybindNo = true
	self:EnableKeydown(true)
	wndControl:Enable(false)
	wndControl:SetText("Press A Key")
end

function KiruTweaks:Button_CaptureClearYes( wndHandler, wndControl, eMouseButton )
	self.Preferences.KeybindYes = nil
	self.Option.Window:FindChild("KeybindYesEditbox"):SetText("")
	if self.Preferences.KeybindNo == nil then
		self.Preferences.YesNo = false
		self.Option.Window:FindChild("Button_YesNo"):SetCheck(false)
		self:RemoveHookYesNo()
	end
end

function KiruTweaks:Button_CaptureClearNo( wndHandler, wndControl, eMouseButton )
	self.Preferences.KeybindNo = nil
	self.Option.Window:FindChild("KeybindNoEditbox"):SetText("")
	if self.Preferences.KeybindYes == nil then
		self.Preferences.YesNo = false
		self.Option.Window:FindChild("Button_YesNo"):SetCheck(false)	
		self:RemoveHookYesNo()
	end
end

function KiruTweaks:Button_ReplyBusy( wndHandler, wndControl, eMouseButton )
	self.Preferences.AutoReplyBusy = wndControl:IsChecked()
end

function KiruTweaks:EditBox_Busy_Changed( wndHandler, wndControl, strText )
	self.Preferences.BusyMessage = strText
end

function KiruTweaks:Button_ReplyCombat( wndHandler, wndControl, eMouseButton )
	self.Preferences.AutoReplyCombat = wndControl:IsChecked()
	self.Option.Window:FindChild("Button_AutoReplyCombatTarget"):Enable(self.Preferences.AutoReplyCombat)
end

function KiruTweaks:EditBox_Combat_Changed( wndHandler, wndControl, strText )
	self.Preferences.CombatMessage = strText
end

function KiruTweaks:Button_ReplyCombatTarget( wndHandler, wndControl, eMouseButton )
	self.Preferences.AutoReplyCombatTarget = wndControl:IsChecked()
end

function KiruTweaks:Round(num, dp)
--rounds a number to specifed decimal places
	local mult = 10^(dp or 0)
	return math.floor(num * mult + 0.5) / mult
end


function KiruTweaks:Button_ShowStartup( wndHandler, wndControl, eMouseButton )
	self.Preferences.ShowStartup = wndControl:IsChecked()
end

function KiruTweaks:Button_HideHouse( wndHandler, wndControl, eMouseButton )
	self.Preferences.HideHouse = wndControl:IsChecked()
	local isHouse = HousingLib.IsHousingWorld()
	local remodel = Apollo.GetAddon("HousingRemodel")
	if isHouse == true and remodel ~= nil and remodel.wndConfigure ~= nil then
		remodel.wndConfigure:Show(not self.Preferences.HideHouse)
		if self.Preferences.HideHouse == true then
			self:AddHookHouseRemodel()
			Print("KiruTweaks: Should be hiding the housing banner now.")
		else
			self:RemoveHookHouseRemodel()
			Print("KiruTweaks: No longer attempting to hide the housing banner.")		
		end
	end	
end

function KiruTweaks:Button_ItemLinkSchem( wndHandler, wndControl, eMouseButton )
	self.Preferences.ItemLinkSchem = wndControl:IsChecked()
	if self.Preferences.ItemLinkSchem then
		local hooked = self:AddHookSchematic()
		if hooked == true then
			Print("KiruTweaks: Tradeskill Schematic items should be linkable.")			
		else
			Print("KiruTweaks: Will attempt to connect to the Tradeskill window when you open it.")
		end
	else
		self:RemoveHookSchematic()
		Print("KiruTweaks: No longer attempting to make Tradeskill Schematic items linkable.")			
	end	
end

function KiruTweaks:Button_AddWhisper( wndHandler, wndControl, eMouseButton )
	self.Preferences.AddWhisper = wndControl:IsChecked()
	if self.Preferences.AddWhisper then
		local hooked = self:AddHookContextMenu()
		if hooked == true then
			Print("KiruTweaks: Should have Whisper for your Account Friends now.")			
		else
			Print("KiruTweaks: There was a problem trying to hook into the context menu.")
		end
	else
		self:RemoveHookContextMenu()
		Print("KiruTweaks: No longer attempting to add Whisper for your Account Friends.")			
	end	
end

-----------------------------------------------------------------------------------------------
-- KiruTweaks Instance
-----------------------------------------------------------------------------------------------
local KiruTweaksInst = KiruTweaks:new()
KiruTweaksInst:Init()
